//#include <glib.h>
//#include <stdio.h>
//#include <sys/types.h>
//#include <sys/socket.h>
//#include <sys/un.h>
//#include <string.h>
//#include <unistd.h>

#include "amp.h"


static GList *joblist = NULL;
static GList *highprio_list = NULL;

static struct packet_functions  *packet_handlers = NULL;

static gboolean running = FALSE;



void cli_init(void) {
    
    packet_handlers = g_malloc0(sizeof(packet_functions));
    return;
}

gint cli_to_serv_conn(void) {
  
    gint fd;
    //gint len;
    gchar *sockfile = NULL;
    gchar *conn_msg = NULL;
    gchar buf[1024];
  
    struct sockaddr_un saddr;

    if ((fd = socket(AF_UNIX, SOCK_STREAM, 0)) == -1) {
        perror("socket()");
        return -1;
    } 

    memset(&saddr,0,sizeof(saddr));
  
    saddr.sun_family = AF_UNIX;
    strcpy(saddr.sun_path,"/tmp/sockfile");

    //len = sizeof(struct sockaddr_un);

    if ((connect(fd, (struct sockaddr *) &saddr, /*len*/ sizeof(struct sockaddr_un) )) == -1) {
        perror("connect()");
        return -2;
    }
  
    conn_msg = g_strdup_printf("CONNECT %s %d",g_get_user_name(), getpid());
  
    //printf("conn_msg = %s\n", conn_msg);
    if ((send(fd, conn_msg, strlen(conn_msg), 0)) <0) {
        perror("send()");
        return -3;
    }
    g_free(conn_msg);
    //printf("Connect request sent to server.\n");
  
    memset(&buf,0,sizeof(buf));
    if ((recv(fd, buf, sizeof(buf)-1, 0)) > 0) {
        //printf("Server: %s",buf);
        if ((strncmp(buf, "ACCEPT", 6)) == 0) {
            //printf("GOT ACCEPT!\n");
            sockfile = strchr(buf, ' ');
            sockfile++;
        }
    }
  
    //printf("Closing old connection.\n");
    close(fd);
  
    if ((fd = socket(AF_UNIX, SOCK_STREAM, 0)) == -1) {
        perror("socket()");
        return -1;
    } 

    memset(&saddr,0,sizeof(saddr));
  
    //sockfile = g_strdup_printf("/tmp/cli.%d",getpid());
    saddr.sun_family = AF_UNIX;
    strcpy(saddr.sun_path,sockfile);
    //g_free(sockfile);
    //len = sizeof(struct sockaddr_un);
    //printf("SOCKFILE = %s\n", sockfile); 
    //printf("strlen = %d\n", strlen(sockfile));
    //printf("strlen un = %d\n", strlen(saddr.sun_path));
    //printf("saddr = '%s'\n",saddr.sun_path);
    //sleep(4);
    if ((connect(fd, (struct sockaddr *) &saddr, /* len */ sizeof(struct sockaddr_un) )) == -1) {
        perror("connect()");
        return -2;
    }

    //printf("New connection established.\n");
    return(fd);
}

void cli_main_loop(gint fd) {

  fd_set                rfds;
  struct packet         *pkt;
  struct timeval        tv;
  
  FD_ZERO(&rfds);
  
  running = TRUE;
  
  while (running) {
      tv.tv_sec = 0;
      tv.tv_usec = 100000;
      FD_SET(fd, &rfds);
      if ((select(fd + 1, &rfds, NULL, NULL, &tv)) > 0) {
          /* packet from server */
          pkt = (struct packet *) g_malloc0(sizeof(struct packet));
          if ((read(fd,&pkt->header,sizeof(struct packetheader))) <= 0)
              break;
          if (pkt->header.length > 0) {
              pkt->data = g_malloc0(pkt->header.length);
              read(fd,pkt->data,pkt->header.length);
          }

          cli_process_pkt(fd,pkt);
          g_free(pkt -> data);
          g_free(pkt);
          
       }
    
  }

}

void cli_process_pkt(gint fd, struct packet *pkt) {
#ifdef DEBUG
    printf("PACKET RECIVIED\n");
    printf("Version: %d\n",pkt->header.version);
    printf("Command: %d\n",pkt->header.cmd);
    printf("Length : %d\n",pkt->header.length);
#endif
    switch (pkt->header.cmd) {
        case S_PONG:
            /* this is only used by clients that need to know when
             * all data from server has been recived. commandline
             * clients for example */
            running = FALSE;
            break;
        case S_SEND_ERROR_MSG:
             if (packet_handlers)
                 if (packet_handlers->cli_errormsg_handler)
                     (*packet_handlers->cli_errormsg_handler)((gchar *) pkt->data);
             break;
        case S_SEND_HP_SONG: {
            struct d_sq *dsqp = pkt -> data;
            highprio_list = cli_add_highprio_song(highprio_list, dsqp -> path, -1, dsqp -> played, &dsqp->info);
            break;
        }
        case S_SEND_JOB_LIST: {
            struct d_jq *djqp = pkt -> data;
            printf("add jobb!\n");
            joblist = add_job(joblist, djqp -> name, djqp -> random, -1);
            break;
        }
        case S_SEND_SONG_QUEUE: {
             struct d_sq *dsqp = pkt -> data;
             cli_add_song_to_job(joblist, dsqp -> jobnr, dsqp -> path, -1, dsqp -> played, &dsqp->info);
             break;
        }
        case S_SEND_CURRENTLY_PLAYING: {
             if (packet_handlers)
                 if (packet_handlers->cli_currplay_handler)
                     (*packet_handlers->cli_currplay_handler)((struct d_sq *) pkt->data);
             break;
        }     
        case S_SEND_TIMEINFO: {
             if (packet_handlers)
                 if (packet_handlers->cli_timeinfo_handler)
                     (*packet_handlers->cli_timeinfo_handler)(*(gint *) pkt->data);
             break;
        }                        
        case S_SEND_LIST_CHANGED: {
             if (packet_handlers)
                 if (packet_handlers->cli_listchanged_handler)
                     (*packet_handlers->cli_listchanged_handler)();
             break;
        }
    
    }

}

GList *cli_get_listp(void) {

    return joblist;
}

GList *cli_get_hplistp(void) {

    return highprio_list;
}

struct packet_functions *cli_get_packet_handler_pointer(void) {
   
    return packet_handlers;
}

void cli_free_lists(void) {

    struct jobqueue *jtmp = NULL;
    /* free high priority list */
    g_list_foreach(highprio_list, (GFunc) free_songqueue_data, NULL);
    g_list_free(highprio_list);
    
    /* free joblist and all sonqueue elements attached to it */
    while (joblist != NULL) {
        jtmp = joblist -> data;
        g_list_foreach(jtmp -> songqueue, (GFunc) free_songqueue_data, NULL);
        g_list_free(jtmp -> songqueue);
        free_jobqueue_data(joblist -> data);
        //joblist = g_list_remove(joblist, joblist -> data);
        joblist = g_list_next(joblist);
    }
    printf("before g_list_free\n");
    g_list_free(joblist);
    if (joblist == NULL)
        printf("joblist r NULL\n");
    
    return;
    
}

