#include "amp.h"



void send_pkt(gint fd, gint command, gpointer data, gint length) {
    
    struct packetheader h;
    
    h.version = PROTOCOL_VERSION;
    h.length = length;
    h.cmd = command;
    write(fd, &h, sizeof(struct packetheader));
    if (data != NULL && length > 0)
        write(fd, data, h.length);
    
}

/* c_ functions are sent from the client to the server, s_ functions
 * from server to client.
 */


void s_send_hp_list(gint fd, GList *hplist) {

    GList *liststart = hplist;
    gint i = g_list_length(hplist);

    if (i == 0)
        return;

    g_list_foreach(hplist, (GFunc) s_send_sq_element, (gpointer) fd);
    
    hplist = liststart;

}     

void s_send_sq_element(gpointer element, gpointer userdata) {

    /* used to send highpriority sq-element */
    
    struct songqueue *sq = element;
    struct d_sq tmp;
    
    gint fd = GPOINTER_TO_INT(userdata);
    
    memset(&tmp,0,sizeof(struct d_sq));
    
    strncpy(tmp.path, sq->path, sizeof(tmp.path)-1);
    tmp.played = sq->played;
    
    memcpy(&tmp.info, sq->info, sizeof(struct songinfo));
    
    send_pkt(fd, S_SEND_HP_SONG, &tmp, sizeof(struct d_sq));
}

void s_pong(gint fd) {

    send_pkt(fd, S_PONG, NULL, 0);
}

void s_send_joblist(gint fd, GList *jlist) {

    GList *liststart = jlist;
    gint i = g_list_length(jlist);

    if (i == 0)
        return;

    g_list_foreach(jlist, (GFunc) s_send_jq_element, (gpointer) fd);
    
    jlist = liststart;

}

void s_send_jq_element(gpointer element, gpointer userdata) {

    struct jobqueue *jq = element;
    struct d_jq tmp;
    
    gint fd = GPOINTER_TO_INT(userdata);
    
    memset(&tmp,0,sizeof(struct d_jq));
    
    strncpy(tmp.name, jq->name, sizeof(tmp.name)-1);
    tmp.random = jq->random;

    send_pkt(fd, S_SEND_JOB_LIST, &tmp, sizeof(struct d_jq));
}

void s_send_songqueue(gint fd, GList *jlist) {

    GList       *liststart = jlist;
    GList  *songliststart;
    
    struct jobqueue  *jtmp;
    struct songqueue *stmp;
    
    gint  i = 0;
     
    while (jlist != NULL) {
        jtmp = jlist -> data;
        songliststart = jtmp -> songqueue;
        while (jtmp -> songqueue != NULL) {
            stmp = jtmp -> songqueue -> data;
            s_send_songqueue_elements(fd, stmp, i);
            jtmp -> songqueue = g_list_next(jtmp -> songqueue);
        }
        i++;
        jtmp -> songqueue = songliststart;
        jlist = g_list_next(jlist);
    }
    jlist = liststart;
}

void s_send_songqueue_elements(gint fd, struct songqueue *sq, gint jobnr) {

    struct d_sq tmp;
    
    memset(&tmp,0,sizeof(struct d_sq));
    strncpy(tmp.path, sq->path, sizeof(tmp.path)-1);
    
    tmp.played = sq->played;
    tmp.jobnr = jobnr;
    
    memcpy(&tmp.info, sq->info, sizeof(struct songinfo));
    
    send_pkt(fd, S_SEND_SONG_QUEUE, &tmp, sizeof(struct d_sq));
}

void s_send_currently_playing(gint fd, struct songqueue *sq) {

    struct d_sq   tmp;

    memset(&tmp,0,sizeof(struct d_sq));
    strncpy(tmp.path, sq->path, sizeof(tmp.path)-1);
    
    tmp.played = sq->played;
    
    memcpy(&tmp.info, sq->info, sizeof(struct songinfo));
    
    send_pkt(fd, S_SEND_CURRENTLY_PLAYING, &tmp, sizeof(struct d_sq));
}

void s_send_time_info(gint fd, gint time) {

    send_pkt(fd, S_SEND_TIMEINFO, &time, sizeof(gint));
}

void s_send_errormsg(gint fd, gchar *msg) {

    send_pkt(fd, S_SEND_ERROR_MSG, msg, strlen(msg));
}

void s_send_list_changed(gint fd) {

    send_pkt(fd, S_SEND_LIST_CHANGED, NULL, 0);
}







void c_addjob(gint fd, gchar *name, gboolean random, gint position) {

    struct d_jobadd tmp;

    memset(&tmp,0,sizeof(struct d_jobadd));
    strncpy(tmp.name, name, sizeof(tmp.name)-1);
    tmp.random = random;
    tmp.position = position;

    send_pkt(fd, C_L_ADD_JOB, &tmp, sizeof(struct d_jobadd));
}
void c_play(gint fd) {

    send_pkt(fd, C_P_PLAY, NULL, 0);
}
void c_stop(gint fd) {

    send_pkt(fd, C_P_STOP, NULL, 0);
}
void c_pause(gint fd) {

    send_pkt(fd, C_P_PAUSE, NULL, 0);
}
void c_seek(gint fd, gint val) {

    send_pkt(fd, C_P_SEEK, &val, sizeof(gint));
}
void c_addsong(gint fd, gint jobnr, gchar *path, gint position) {

    struct d_songadd tmp;

    memset(&tmp,0,sizeof(struct d_songadd));
    strncpy(tmp.path, path, sizeof(tmp.path)-1);
    tmp.played = FALSE;
    tmp.jobnr = jobnr;
    tmp.position = position;

    send_pkt(fd, C_L_ADD_SONG, &tmp, sizeof(struct d_songadd));
}

void c_get_hp_list(gint fd) {

    send_pkt(fd, C_HPL_GET_LIST, NULL, 0);
}

void c_ping(gint fd) {
    
    send_pkt(fd, C_PING, NULL, 0);
}

void c_get_joblist(gint fd) {
    
    send_pkt(fd, C_L_GET_JOB_LIST, NULL, 0);
}

void c_get_songqueue(gint fd) {

    send_pkt(fd, C_L_GET_SONG_QUEUE, NULL, 0);
}

void c_get_all_lists(gint fd) {

   c_get_hp_list(fd);
   c_get_joblist(fd);
   c_get_songqueue(fd);
}   

void c_add_hp_song(gint fd, gchar *path, gint position) {

    struct d_songadd tmp;

    memset(&tmp,0,sizeof(struct d_songadd));
    strncpy(tmp.path, path, sizeof(tmp.path)-1);
    tmp.played = FALSE;
    tmp.jobnr = 0; /* not used here */
    tmp.position = position;

    send_pkt(fd, C_HPL_ADD_SONG, &tmp, sizeof(struct d_songadd));

}

void c_remove_job(gint fd, gint jobnr) {

    send_pkt(fd, C_L_REMOVE_JOB, &jobnr, sizeof(gint));
}

void c_serv_shutdown(gint fd) {

    send_pkt(fd, C_SHUTDOWN, NULL, 0);
}

void c_set_pcm_volume(gint fd, guint volume) {

    send_pkt(fd, C_S_SET_PCM, &volume, sizeof(gint));
}

void c_request_timeinfo(gint fd) {

    send_pkt(fd, C_S_REQ_TIMEINFO, NULL, 0);
}

void c_abort_timeinfo(gint fd) {

    send_pkt(fd, C_S_ABORT_TIMEINFO, NULL, 0);
}
