#define PROTOCOL_VERSION        1

#define MAX_PATH_LENGTH         1024
#define MAX_JOB_NAME_LENGTH     128


enum {
    /* commands sent from client to server */
    C_PING, C_SHUTDOWN, C_P_PLAY, C_P_STOP, C_P_PAUSE, C_P_SEEK,
    C_S_SET_PCM, C_S_REQ_TIMEINFO, C_S_ABORT_TIMEINFO, C_L_ADD_JOB,
    C_L_ADD_SONG, C_L_REMOVE_JOB, C_L_REMOVE_SONG, C_HPL_GET_LIST,
    C_L_GET_JOB_LIST, C_L_GET_SONG_QUEUE, C_HPL_ADD_SONG,
};

enum {
    /* commands sent from server to ckient */
    S_PONG, S_SEND_ERROR_MSG, S_SEND_HP_SONG, S_SEND_JOB_LIST,
    S_SEND_SONG_QUEUE, S_SEND_CURRENTLY_PLAYING, S_SEND_TIMEINFO,
    S_SEND_LIST_CHANGED,
};


typedef struct packetheader {
    gint version;
    gint length;
    gint cmd;
} packetheader;

typedef struct packet {
    packetheader header;
    gpointer     data;
} packet; 

/* d_ structs are used to send data between client and server */

typedef struct d_jobadd {
    gchar       name[MAX_JOB_NAME_LENGTH];
    gboolean    random;
    gint        position;
} d_jobadd;

typedef struct d_songadd {
    gchar       path[MAX_PATH_LENGTH];
    gint        jobnr;
    gboolean    played;
    gint        position;
} d_songadd;

typedef struct d_sq {
    gchar           path[MAX_PATH_LENGTH];
    gboolean        played;
    gint            jobnr;
    struct songinfo info;
} d_sq;

typedef struct d_jq {
    gchar       name[MAX_JOB_NAME_LENGTH];
    gboolean    random;
} d_jq;

  

void send_pkt(gint fd, gint command, gpointer data, gint length);

void c_addjob(gint fd, gchar *name, gboolean random, gint position);
void c_addsong(gint fd, gint jobnr, gchar *path, gint position);
void c_play(gint fd);
void c_stop(gint fd);
void c_pause(gint fd);
void c_seek(gint fd, gint val);
void c_get_hp_list(gint fd);
void c_ping(gint fd);
void c_get_joblist(gint fd);
void c_get_songqueue(gint fd);
void c_get_all_lists(gint fd); 
void c_add_hp_song(gint fd, gchar *path, gint position);
void c_remove_job(gint fd, gint jobnr);
void c_serv_shutdown(gint fd);
void c_set_pcm_volume(gint fd, guint volume);
void c_request_timeinfo(gint fd);
void c_abort_timeinfo(gint fd);



void s_send_hp_list(gint fd, GList *hplist);
void s_send_sq_element(gpointer element, gpointer userdata);
void s_pong(gint fd);
void s_send_joblist(gint fd, GList *jlist);
void s_send_jq_element(gpointer element, gpointer userdata);
void s_send_songqueue(gint fd, GList *jlist);
void s_send_songqueue_elements(gint fd, struct songqueue *sq, gint jobnr);
void s_send_currently_playing(gint fd, struct songqueue *sq);
void s_send_time_info(gint fd, gint time);
void s_send_errormsg(gint fd, gchar *msg);
void s_send_list_changed(gint fd);

