#include <glib.h>
#include <getopt.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include "amp.h"

static gint     insert = -1; /* negative value will add in the end of the list */
static gint     playorder = 0; /* 0 for sequence, 1 for random */
static gint     jobnr = 0;
static gint     rmjob = 0;

static gboolean addjob = FALSE;
static gchar    *jobname = NULL;

static gboolean addsong = FALSE;                             
static gboolean addhpsong = FALSE;
static guint    songqnr = 0;
static gboolean listall = FALSE;
static gboolean listhp = FALSE;
static gboolean listjob = FALSE;
static gboolean listsong = FALSE;
static gboolean removejob = FALSE;


static gint     seekval = 0;
static gboolean seek = FALSE;
static gboolean play = FALSE;
static gboolean stop = FALSE;
static gboolean paus = FALSE;
static gboolean skip_to_next = FALSE;
static gboolean setvolume = FALSE;
static gint     volume;
static gboolean killserv = FALSE;
static gboolean timeinfo = FALSE;

static gint     total_secs = 0;

static struct packet_functions  *packet_handlers = NULL;

void cmdline_print_jobs(void);
void cmdline_print_job(guint jobnr);
void cmdline_print_all(void);
void cmdline_print_hp(void);
void cmdline_print_help(void);
void cmdline_print_time(gint sec);
void cmdline_print_currplay(struct d_sq *dsq);
void cmdline_temp(void);

int main(int argc, char **argv) {
   
   gint optindex = 0;
   gint ret;
   
   static struct option long_options[] = {
       {"addjob"        , required_argument, 0          , 'j'},
       {"addsong"       , optional_argument, 0          , 's'},
       {"list"          , required_argument, 0          , 'l'},
       {"insert"        , required_argument, 0          , 'i'},
       {"rmjob"         , required_argument, 0          , 'r'},
       {"play"          , no_argument      , 0          , 'P'},
       {"stop"          , no_argument      , 0          , 'S'},
       {"pause"         , no_argument      , 0          , 'p'},
       {"seek"          , required_argument, 0          , 'g'},
       {"volume"        , required_argument, 0          , 'v'},
       {"kill"          , no_argument      , 0          , 'k'},
       {"timeinfo"      , no_argument      , 0          , 't'},
       {"help"          , no_argument      , 0          , 'h'},
       {"next"          , no_argument      , 0          , 'n'},
       {"seq"           , no_argument      , &playorder , 0  },
       {"rnd"           , no_argument      , &playorder , 1  },
       {0               , 0                , 0          , 0  }
   };
   
   
   while (1) {
       
       ret = getopt_long(argc, argv, "j:s:l:i:r:PSpg:v:ikthn", long_options, &optindex);

       if (ret == -1) /* no options left */
           break;
       
       switch (ret) {
           case '?':
               printf("\nTry `cmdline --help' for information.\n");
               exit(-1);
               break;
           case 'j':
               addjob = TRUE;
               jobname = g_strdup(optarg);
               break;
           case 's':
               if (optarg[0] == 'h' && optarg[1] == 'p')
                   addhpsong = TRUE;
               else if (optarg[0] == 'l') {
                   jobnr = -1;
                   addsong = TRUE;
               }
               else { 
                   jobnr = atoi(optarg)-1;              
                   addsong = TRUE;
               }
               break;
           case 'l': 
               if ((strncmp(optarg,"all",3)) == 0 || optarg == NULL) 
                   listall = TRUE;
               else if ((strncmp(optarg,"hp",2)) == 0)
                   listhp = TRUE;
               else if ((strncmp(optarg,"job",3)) == 0)
                   listjob = TRUE;
               else {
                   listsong = TRUE;
                   songqnr = atoi(optarg)-1;
               }
               break;
           case 'i':
               insert = atoi(optarg)-1;
               break;
           case 'r':
               removejob = TRUE;
               rmjob = atoi(optarg)-1;
               break;
           case 'P':
               play = TRUE;
               break;
           case 'S':
               stop = TRUE;
               break;
           case 'p':
               paus = TRUE;
               break;
           case 'g':
               seek = TRUE;
               seekval = atoi(optarg);
               break;
           case 'v':
               setvolume = TRUE;
               volume = atoi(optarg);
               break;
           case 'k':
               killserv = TRUE;
               break;
           case 't':
               timeinfo = TRUE;
               break;
           case 'h':
               cmdline_print_help();
               exit(0);
               break;
           case 'n':
               skip_to_next = TRUE;
               break;
       }
   }
   
   gint fd;

   cli_init();
   packet_handlers = cli_get_packet_handler_pointer();
   packet_handlers->cli_timeinfo_handler = &cmdline_print_time;
   packet_handlers->cli_currplay_handler = &cmdline_print_currplay;
   packet_handlers->cli_listchanged_handler = &cmdline_temp;
    
   if ((fd = cli_to_serv_conn()) < 0) {
       printf("Unable to connect!\n");
       exit(-1);
   }

   
   if (killserv)
       c_serv_shutdown(fd);
   
   if (timeinfo) {
       c_request_timeinfo(fd);
       cli_main_loop(fd);
   }
       
   if (addjob) {
       c_addjob(fd, jobname, (gboolean) playorder, insert);
       g_free(jobname);
   }

   if (addsong || addhpsong) {
       gchar *pathtmp;
       while (optind < argc) { 
           if (argv[optind][0] == '/')
               pathtmp = g_strdup(argv[optind]);
           else
               pathtmp = g_strdup(g_build_path("/",g_get_current_dir(),argv[optind],NULL));
       
           optind++;
           if (addsong)
               c_addsong(fd, jobnr, pathtmp, insert);
           else
               c_add_hp_song(fd, pathtmp, insert);
           g_free(pathtmp);
       }
   }
   
   if (play) 
       c_play(fd);
   else if (stop)
       c_stop(fd);
   else if (paus)
       c_pause(fd);
   else if (seek)
       c_seek(fd,seekval);
   else if (skip_to_next)
       c_seek(fd,100); /* seek to end */     
       
   if (setvolume)
       c_set_pcm_volume(fd, volume);
   
   if (removejob)
       c_remove_job(fd, rmjob);

   
   if (listjob) {
       c_get_all_lists(fd);
       c_ping(fd);
       cli_main_loop(fd);
       cmdline_print_jobs();
   }    
   else if (listall) {
       c_get_all_lists(fd);
       c_ping(fd);
       cli_main_loop(fd);
       cmdline_print_all();
   }    
   else if (listhp) {
       c_get_all_lists(fd);
       c_ping(fd);
       cli_main_loop(fd);
       cmdline_print_hp();
   }
   else if (listsong) {
       c_get_all_lists(fd);
       c_ping(fd);
       cli_main_loop(fd);
       cmdline_print_job(songqnr);
   }
}

void cmdline_print_jobs(void) {

    GList *tmp = cli_get_listp();
    struct jobqueue *jtmp;
    gint nr = 0;
    struct jobstatus jstat;
    
    while (tmp != NULL) {
        jtmp = tmp -> data;
        jstat.total_songs = 0;
        jstat.played = 0;
        get_current_jobstatus(tmp,&jstat);
        printf("%3d %s (%.2d/%.2d): %s\n", nr+1, jtmp -> random ? "[RND]" : "[SEQ]" ,jstat.played, jstat.total_songs, jtmp -> name);
        tmp = g_list_next(tmp);
        nr++;
    }
}

void cmdline_print_job(guint jobnr) {

    GList *tmp = cli_get_listp();
    
    gint nr = 0;
    struct jobqueue *jtmp;
    struct songqueue *stmp;
    struct jobstatus jstat;
 
    tmp = g_list_nth(tmp, jobnr);
    
    if (tmp == NULL)
        return;
    
    jtmp = tmp -> data;
    jstat.total_songs = 0;
    jstat.played = 0;
    get_current_jobstatus(tmp,&jstat);
    printf("-(%.2d)----(%-42.42s)---------(%s)---(%.2d/%.2d)--\n", nr+1, jtmp -> name, jtmp -> random ? "RND" : "SEQ", jstat.played, jstat.total_songs);
    GList *songliststart = jtmp -> songqueue;
    while (jtmp -> songqueue != NULL) {
        stmp = jtmp -> songqueue -> data;
        if (stmp->info->artist[0] != '\0' && stmp->info->title[0] != '\0')
            printf("    [%s]  %-20.20s - %-27.27s   %.2d:%.2d  %.10s\n", stmp -> played ? "X" : " ", stmp->info->artist, stmp->info->title, stmp->info->minutes, stmp->info->seconds, stmp->info->genre);
        else
            printf("    [%s]  %-51.51s  %.2d:%.2d  %.10s\n", stmp -> played ? "X" : " ", g_path_get_basename(stmp -> path), stmp->info->minutes, stmp->info->seconds, stmp->info->genre);

            
        jtmp -> songqueue = g_list_next(jtmp -> songqueue);
    }
    jtmp -> songqueue = songliststart;
    putchar('\n');
}
    
void cmdline_print_all(void) {

    cmdline_print_hp();
    
    GList *tmp = cli_get_listp();

    gint nr = 0;
    struct jobqueue *jtmp;
    struct songqueue *stmp;
    struct jobstatus jstat;
    
    while (tmp != NULL) {
        jtmp = tmp -> data;
        jstat.total_songs = 0;
        jstat.played = 0;
        get_current_jobstatus(tmp,&jstat);
        printf("-(%.2d)----(%-42.42s)---------(%s)---(%.2d/%.2d)--\n", nr+1, jtmp -> name, jtmp -> random ? "RND" : "SEQ", jstat.played, jstat.total_songs);
        GList *songliststart = jtmp -> songqueue;
        while (jtmp -> songqueue != NULL) {
            stmp = jtmp -> songqueue -> data;
            if (stmp->info->artist[0] != '\0' && stmp->info->title[0] != '\0')
                printf("    [%s]  %-20.20s - %-27.27s   %.2d:%.2d  %.10s\n", stmp -> played ? "X" : " ", stmp->info->artist, stmp->info->title, stmp->info->minutes, stmp->info->seconds, stmp->info->genre);
            else
                printf("    [%s]  %-51.51s  %.2d:%.2d  %.10s\n", stmp -> played ? "X" : " ", g_path_get_basename(stmp -> path), stmp->info->minutes, stmp->info->seconds, stmp->info->genre);

            
            jtmp -> songqueue = g_list_next(jtmp -> songqueue);
        }
        jtmp -> songqueue = songliststart;
        tmp = g_list_next(tmp);
        nr++;
        putchar('\n');
    }
}

void cmdline_print_hp(void) {

   GList *tmp = cli_get_hplistp();
   struct songqueue *stmp;
   struct jobstatus jstat;
       
   get_current_hpstatus(tmp, &jstat);
   if (tmp == NULL)
       return;
   
   printf("---------(%-42.42s)---------(%s)---(%.2d/%.2d)--\n", "High Priority List", "SEQ", jstat.played, jstat.total_songs);
   
   while (tmp != NULL) {
       stmp = tmp -> data;
       if (stmp->info->artist[0] != '\0' && stmp->info->title[0] != '\0')
           printf("    [%s]  %-20.20s - %-27.27s   %.2d:%.2d  %.10s\n", stmp -> played ? "X" : " ", stmp->info->artist, stmp->info->title, stmp->info->minutes, stmp->info->seconds, stmp->info->genre);
       else
           printf("    [%s]  %-51.51s  %.2d:%.2d  %.10s\n", stmp -> played ? "X" : " ", g_path_get_basename(stmp -> path), stmp->info->minutes, stmp->info->seconds, stmp->info->genre);
       tmp = g_list_next(tmp);
   }
   putchar('\n');
}
  
void cmdline_print_time(gint sec) {
    
    gchar progressbar[60];
    gint  h_tot=0, m_tot=0, s_tot=0;
    gint  h=0, m=0, s=0;
    gint  counter, stars=0;
    gchar *p;
    
    putchar('\r');
    memset(&progressbar,0,sizeof(progressbar));
    
    stars = (sec * 50) / total_secs;
        
    p = progressbar;
    *p = '[';
    for (counter = 0; counter <= 50; counter++) {
        p++;
        if (counter < stars)
            *p = '*';
        else
            *p = ' ';
    }
    *p++ = ']';

    h_tot = (total_secs/(60*60))%24;
    m_tot = (total_secs/60)%60;
    s_tot = total_secs%60;
    
    h = (sec/(60*60))%24;
    m = (sec/60)%60;
    s = sec%60;

    if (h_tot == 0) 
        printf("%.2d:%.2d   %s %.2d:%.2d", m, s, progressbar, m_tot, s_tot);
    else
        printf("%.2d:%.2d:%.2d   %s %.2d:%.2d:%.2d", h, m, s, progressbar, h_tot, m_tot, s_tot);
    //printf("TIME: %d",i);
    fflush(stdout);
    return;
}

void cmdline_print_currplay(struct d_sq *dsq) {

    printf("\033[2J\033[H");
    if (dsq->info.title[0] != '\0' && dsq->info.artist[0] != '\0') 
        printf("%s - %s\n", dsq->info.artist, dsq->info.title);
    else
        printf("%s\n", g_path_get_basename(dsq -> path));
    
    total_secs = (dsq->info.minutes*60)+dsq->info.seconds;
    
    putchar('\n');
    //printf("Title : %s\n", dsq->info.title);
    //printf("Artist: %s\n", dsq->info.artist);
    //printf("Time  : %.2d:%.2d\n\n", dsq->info.minutes, dsq->info.seconds);
    return;
}

void cmdline_print_help(void) {

    printf("Usage:  cmdline [list options] [file]\n");
    printf("        cmdline [play options]\n\n");
    printf("Player options:\n");
    printf("  -P, --play                        play\n");
    printf("  -S, --stop                        stop\n");
    printf("  -p, --pause                       pause\n");
    printf("  -g, --seek <0-100>                seek to position\n");
    printf("  -n, --next                        jump to next song\n");
    printf("  -v, --volume <0-100>              set pcm volume\n");
    printf("  -t, --timeinfo                    information about currently playing song\n"); 
    printf("  -k, --kill                        shutdown server\n\n");
    printf("List options:\n");
    printf("  -j, --addjob <jobname>            add a new job\n");
    printf("      --rnd                         used with addjob, tag job as random play\n");
    printf("      --seq                         tag job as sequence play (default)\n");
    printf("  -s, --addsong hp | l | <jobnr>    hp adds songs to the high priority list\n");
    printf("                                    l adds songs to the last job\n");
    printf("                                    <jobnr> adds songs to specified job number\n");
    printf("  -i, --insert <position>           used with addjob and addsong\n");
    printf("                                    with addjob, inserts job at position\n");
    printf("                                    with addsong, inserts song at position\n");
    printf("  -l, --list all | hp | jobs | <nr> all prints all lists\n");
    printf("                                    hp prints the high priority list\n");
    printf("                                    jobs prints job list\n");
    printf("                                    <nr> prints songs in specified job number\n");
    printf("  -r, --rmjob <jobnr>               removes job at specified number\n");
    
    putchar('\n');
}
   
void cmdline_temp(void) {

    printf("\nfkjsflkdsfdslkjflkjdsljfkds\n");
}

