#include <gtk/gtk.h>
#include <dirent.h>
#include "../amp.h"

/* enum used on the treeview with the lists fetched from server */

enum {
    NUMBER_COLUMN,
    SONG_COLUMN,
    TIME_COLUMN,
    PLAYED_COLUMN,
    SQ_PTR_COLUMN,
    
    VISIBLE_PL_COLUMN,
    ACTIVATABLE_COLUMN,
    LIST_N_COLUMNS
};

/* used with the treeview over the filesystem */

enum {
    FILE_NAME_COLUMN,
    ABS_PATH_COLUMN,
    
    FILE_N_COLUMNS
};

static GtkWidget        *window;
static gint             fd;

static GtkWidget        *list_tree;
static GtkWidget        *file_tree;

static struct packet_functions  *packet_handlers = NULL;


    
static void cb_destroy(GtkWidget *widget, gpointer data);
static gboolean cb_delete_event(GtkWidget *widget, GdkEvent  *event, gpointer data);

GtkWidget *create_list_tree(void);
GtkWidget *create_file_tree(void);
void fill_list_store_with_data(GtkTreeStore *store);
void fill_file_store_with_data(gchar *path, GtkTreeIter *iter, GtkTreeStore *store);
GtkWidget *create_menubar(void);
GtkWidget *create_list_popup_menu(void);
GtkWidget *create_file_popup_menu(void);

void foreach_add_job(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data);

void update_list_tree(void);



/* callbacks */
static gint cb_popup_handler(GtkWidget *widget, GdkEvent *event);
static gint cb_add_job(GtkWidget *widget, gpointer data);

static void cb_destroy(GtkWidget *widget, gpointer data) {
        
    gtk_main_quit ();
}

static gboolean cb_delete_event(GtkWidget *widget, GdkEvent  *event, gpointer data) {
    

    return FALSE;
    
}

gpointer gtkcli_client_loop(gpointer data) {
    
    cli_init();
    
    packet_handlers = cli_get_packet_handler_pointer();
    packet_handlers->cli_listchanged_handler = &update_list_tree;
    
    fd = cli_to_serv_conn();
    //cli_free_lists(); 
    
    c_get_all_lists(fd);
    //c_ping(fd);
    cli_main_loop(fd);
    printf("MAIN LOOP EXITS\n");
    g_thread_exit(NULL);
    return 0;
} 
    
int main(int argc, char **argv) {

    GtkWidget        *list_scrollwin;
    GtkWidget        *file_scrollwin;
    GtkWidget        *menu_bar;
    GtkWidget        *list_popup_menu;
    GtkWidget        *file_popup_menu;
    GtkWidget        *tree_hbox;
    GtkWidget        *vbox;
    
    
    gtk_init (&argc, &argv);

    g_thread_init(NULL);
    g_thread_create(&gtkcli_client_loop, NULL, FALSE, NULL);   
    
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    g_signal_connect(G_OBJECT(window), "delete_event", G_CALLBACK(cb_delete_event), NULL);
    g_signal_connect(G_OBJECT(window), "destroy", G_CALLBACK(cb_destroy), NULL);

    gtk_window_set_default_size(GTK_WINDOW (window), 800, 500);
    //gtk_window_set_decorated(GTK_WINDOW (window), 0);
    
    menu_bar = create_menubar();
    
    file_tree = create_file_tree();
    list_tree = create_list_tree();
    
    vbox = gtk_vbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER(window), vbox);
    gtk_widget_show(vbox);
    
    gtk_box_pack_start(GTK_BOX(vbox), menu_bar, FALSE, TRUE, 0);
    //gtk_widget_show(menu_bar);
    tree_hbox = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_end(GTK_BOX(vbox), tree_hbox, TRUE, TRUE, 0);
    //gtk_container_add(GTK_CONTAINER(window), hbox);
    gtk_widget_show(tree_hbox);
    
    /* list tree */
    list_scrollwin = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (list_scrollwin), GTK_SHADOW_ETCHED_IN);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (list_scrollwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_widget_show(list_scrollwin);
    
    gtk_box_pack_start(GTK_BOX(tree_hbox), list_scrollwin, TRUE, TRUE, 0);
    gtk_container_add(GTK_CONTAINER (list_scrollwin), list_tree);
    
    list_popup_menu = create_list_popup_menu();
    g_signal_connect_swapped(GTK_OBJECT(list_tree), "button_press_event", G_CALLBACK(cb_popup_handler), GTK_OBJECT(list_popup_menu));
    
    gtk_widget_show(list_tree);
    /* file tree */
    file_scrollwin = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (file_scrollwin), GTK_SHADOW_ETCHED_IN);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (file_scrollwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_widget_show(file_scrollwin);
    
    gtk_box_pack_end(GTK_BOX(tree_hbox), file_scrollwin, TRUE, TRUE, 0);
    gtk_container_add(GTK_CONTAINER (file_scrollwin), file_tree);
     
    file_popup_menu = create_file_popup_menu();
    g_signal_connect_swapped(GTK_OBJECT(file_tree), "button_press_event", G_CALLBACK(cb_popup_handler), GTK_OBJECT(file_popup_menu));
    
    
    gtk_widget_show(file_tree);
    

    
    //g_signal_connect(list_tree, "realize", G_CALLBACK (gtk_tree_view_expand_all), NULL);
    gtk_widget_show(window);
        
    gtk_main();
   
    return 0;

}

GtkWidget *create_list_tree(void) {

    GtkTreeStore        *store;
    GtkWidget           *tree;
    GtkTreeViewColumn   *column;
    GtkCellRenderer     *renderer;

    store = gtk_tree_store_new(LIST_N_COLUMNS, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN,
                               G_TYPE_POINTER, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN);

    fill_list_store_with_data(store);

    tree = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));

    g_object_unref(G_OBJECT(store));
    renderer = gtk_cell_renderer_text_new();
    g_object_set(G_OBJECT(renderer), "foreground", "black", NULL);
    
    column = gtk_tree_view_column_new_with_attributes("Nr", renderer, 
                                                      "text", NUMBER_COLUMN, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);
    
    column = gtk_tree_view_column_new_with_attributes("Name", renderer, "text", SONG_COLUMN, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);

    column = gtk_tree_view_column_new_with_attributes("Time", renderer,
                                                      "text", TIME_COLUMN, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);
    
    renderer = gtk_cell_renderer_toggle_new();
    column = gtk_tree_view_column_new_with_attributes("Played", renderer,
                                                      "active", PLAYED_COLUMN,
                                                      "visible", VISIBLE_PL_COLUMN,
                                                      "activatable", ACTIVATABLE_COLUMN, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);
    gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(tree)), GTK_SELECTION_MULTIPLE);
    
    return tree; 

}

void fill_list_store_with_data(GtkTreeStore *store) {

    GList *list = cli_get_listp();
    GList *hplist = cli_get_hplistp();
    
    struct jobqueue     *jtmp;
    struct songqueue    *stmp;
    gint                jobnr = 1;
    gint                songnr = 1;
    GtkTreeIter         jobiter, songiter;
    
    /* high priority list */
    if (hplist != NULL) {
        gtk_tree_store_append(store, &jobiter, NULL);
        gtk_tree_store_set(store, &jobiter,
                           NUMBER_COLUMN, 0, 
                           SONG_COLUMN, "High Priority",
                           TIME_COLUMN, FALSE,
                           PLAYED_COLUMN, FALSE,
                           SQ_PTR_COLUMN, FALSE,
                           VISIBLE_PL_COLUMN, FALSE,
                           ACTIVATABLE_COLUMN, FALSE, -1);
        while (hplist != NULL) {
            stmp = hplist -> data;
            gtk_tree_store_append(store, &songiter, &jobiter);
            gtk_tree_store_set(store, &songiter,
                              NUMBER_COLUMN, songnr++,
                              SONG_COLUMN, (stmp->info->artist && stmp->info->title) ?
                              g_strdup_printf("%s - %s", stmp->info->artist, stmp->info->title)
                              : g_path_get_basename(stmp->path),
                              TIME_COLUMN,
                              g_strdup_printf("%.2d:%.2d",stmp->info->minutes, stmp->info->seconds),
                              PLAYED_COLUMN, stmp->played,
                              SQ_PTR_COLUMN, stmp,
                              VISIBLE_PL_COLUMN, TRUE,
                              ACTIVATABLE_COLUMN, TRUE, -1);
            hplist = g_list_next(hplist);
        }
        songnr = 1;
    }
    
    /* normal list */
    
    while (list != NULL) {
        jtmp = list -> data;
        gtk_tree_store_append(store, &jobiter, NULL);
        gtk_tree_store_set(store, &jobiter,
                           NUMBER_COLUMN, jobnr++,
                           SONG_COLUMN, g_strdup_printf("[%s] %s", jtmp->random ? "RND" : "SEQ", jtmp->name),
                           TIME_COLUMN, FALSE,
                           PLAYED_COLUMN, FALSE,
                           SQ_PTR_COLUMN, FALSE,
                           VISIBLE_PL_COLUMN, FALSE,
                           ACTIVATABLE_COLUMN, FALSE, -1);
        //printf("Job: %s\n", jtmp->name);
        GList *songliststart = jtmp -> songqueue;
        while (jtmp -> songqueue != NULL) {
            stmp = jtmp -> songqueue -> data;
            gtk_tree_store_append(store, &songiter, &jobiter);
            gtk_tree_store_set(store, &songiter,
                               NUMBER_COLUMN, songnr++,
                               SONG_COLUMN, (stmp->info->artist && stmp->info->title) ?
                               g_strdup_printf("%s - %s", stmp->info->artist, stmp->info->title) :
                               g_path_get_basename(stmp->path),
                               TIME_COLUMN,                                    
                               g_strdup_printf("%.2d:%.2d",stmp->info->minutes, stmp->info->seconds),
                               PLAYED_COLUMN, stmp->played,
                               SQ_PTR_COLUMN, stmp,
                               VISIBLE_PL_COLUMN, TRUE,
                               ACTIVATABLE_COLUMN, TRUE, -1);
        jtmp -> songqueue = g_list_next(jtmp -> songqueue);
        //printf("Song: %s\n",g_path_get_basename(stmp->path));
        }
        songnr = 1;
        jtmp -> songqueue = songliststart;
        list = g_list_next(list);
    }

}

 

GtkWidget *create_file_tree(void) {

    GtkTreeStore        *store;
    GtkWidget           *tree;
    GtkTreeViewColumn   *column;
    GtkCellRenderer     *renderer;

    store = gtk_tree_store_new(FILE_N_COLUMNS, G_TYPE_STRING, G_TYPE_STRING);

    //fill_file_store_with_data(store);
    //recursive_filetree_walk("/home/erikw/mp3new/", NULL, store);
    fill_file_store_with_data("/home/erikw/mp3new/", NULL, store);
    //fill_file_store_with_data("/home/erikw/mp3new/", NULL, store);
    //fill_file_store_with_data("/mnt/slave/All/A/", NULL, store);

    tree = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));

    g_object_unref(G_OBJECT(store));
    renderer = gtk_cell_renderer_text_new();
    g_object_set(G_OBJECT(renderer), "foreground", "black", NULL);
    
    column = gtk_tree_view_column_new_with_attributes("File", renderer, 
                                                      "text", FILE_NAME_COLUMN, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);
    
    //gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(tree), TRUE);
    gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(tree)), GTK_SELECTION_MULTIPLE);
    
    return tree; 

}

void fill_file_store_with_data (gchar *path, GtkTreeIter *iter, GtkTreeStore *store) {
    DIR                 *dir;
    struct dirent       *dent;
    GtkTreeIter         subiter;
    
    printf("path = %s\n", path);
    
    
    dir = opendir(path);
    
    while ((dent = readdir(dir)) != NULL) {
        if (dent->d_name[0] != '.') {
            /* ignore everything starting with a . */
            if ((dent->d_type == DT_DIR)) {
                /* a directory */
                gtk_tree_store_append(store, &subiter, iter);
                gtk_tree_store_set(store, &subiter,
                        FILE_NAME_COLUMN, dent->d_name,
                        ABS_PATH_COLUMN, g_build_path("/", path, dent->d_name, NULL),
                        -1);
                //recursive_filetree_walk(g_build_path("/", path, dent->d_name, NULL), &subiter, store);        
                fill_file_store_with_data(g_build_path("/", path, dent->d_name, NULL), &subiter, store);
                //printf("%s\n",dent->d_name);

            } else if ((dent->d_type == DT_REG)) {
                /* a file */
                gtk_tree_store_append(store, &subiter, iter);
                gtk_tree_store_set(store, &subiter,
                        FILE_NAME_COLUMN, dent->d_name,
                        ABS_PATH_COLUMN, g_build_path("/", path, dent->d_name, NULL),
                        -1);
            }
    
        }
    }
}

GtkWidget *create_menubar(void) {

    GtkWidget   *menu_bar;
    GtkWidget   *file_menu;
    GtkWidget   *file_item;
    GtkWidget   *quit_item;
    
    file_menu = gtk_menu_new();
    
    quit_item = gtk_menu_item_new_with_label("Quit");
    gtk_menu_shell_append(GTK_MENU_SHELL(file_menu), quit_item);
    g_signal_connect_swapped(G_OBJECT(quit_item), "activate", G_CALLBACK(cb_destroy), (gpointer) "file.quit");
    gtk_widget_show (quit_item);
    
    menu_bar = gtk_menu_bar_new();
    gtk_widget_show(menu_bar);
    
    file_item = gtk_menu_item_new_with_label("File");
    gtk_widget_show (file_item);
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(file_item), file_menu);
    
    gtk_menu_bar_append(GTK_MENU_BAR(menu_bar), file_item);
    //gtk_menu_item_right_justify(GTK_MENU_ITEM(file_menu));

    return menu_bar;
    
}

GtkWidget *create_list_popup_menu(void) {

    GtkWidget   *list_popup_menu;
    GtkWidget   *addjob_item;
    
    list_popup_menu = gtk_menu_new();
    
    addjob_item = gtk_menu_item_new_with_label("Add new job");
    gtk_menu_shell_append(GTK_MENU_SHELL(list_popup_menu), addjob_item);
    g_signal_connect_swapped(G_OBJECT(addjob_item), "activate", G_CALLBACK(cb_add_job), (gpointer) "add.job");
    gtk_widget_show(addjob_item);

    return list_popup_menu;

}

GtkWidget *create_file_popup_menu(void) {

    GtkWidget   *file_popup_menu;
    GtkWidget   *addjob_item;
    
    file_popup_menu = gtk_menu_new();
    
    addjob_item = gtk_menu_item_new_with_label("Add new job");
    gtk_menu_shell_append(GTK_MENU_SHELL(file_popup_menu), addjob_item);
    g_signal_connect_swapped(G_OBJECT(addjob_item), "activate", G_CALLBACK(cb_add_job), (gpointer) "add.job");
    gtk_widget_show(addjob_item);

    return file_popup_menu;

}

static gint cb_popup_handler(GtkWidget *widget, GdkEvent *event) {
    
    GtkMenu *menu;
    GdkEventButton *event_button;

    g_return_val_if_fail (widget != NULL, FALSE);
    g_return_val_if_fail (GTK_IS_MENU (widget), FALSE);
    g_return_val_if_fail (event != NULL, FALSE);

    menu = GTK_MENU(widget);

    if (event->type == GDK_BUTTON_PRESS) {
        event_button = (GdkEventButton *) event;
        if (event_button->button == 3) {
            gtk_menu_popup (menu, NULL, NULL, NULL, NULL, event_button->button, event_button->time);
            return TRUE;
        }
    }
            
        
    printf("POPUP!\n");

    return FALSE;
}

static gint cb_add_job(GtkWidget *widget, gpointer data) {

    printf("Add job!\n");
    gtk_tree_selection_selected_foreach(gtk_tree_view_get_selection(GTK_TREE_VIEW(file_tree)), foreach_add_job, (gpointer) gtk_tree_view_get_model(GTK_TREE_VIEW(file_tree)));
     
    return FALSE;

}
void foreach_add_job(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data) {

    gchar *tmp;
    
    gtk_tree_model_get(model, iter, ABS_PATH_COLUMN, &tmp, -1);
    
    printf("%s\n", tmp);
    g_free(tmp);
    return;
}

void update_list_tree(void) {

//    c_get_all_lists(fd); 
//    list_tree = create_list_tree();

//    gtk_widget_show_all(window);
    
    GtkTreeStore        *store;
    //GtkWidget           *tree;
    //GtkTreeViewColumn   *column;
    //GtkCellRenderer     *renderer;

    printf("update_list_tree()\n");
    
    
    cli_free_lists();
    printf("free_lists() complete!\n");
    
    print_hierachy(cli_get_listp()); 
    c_get_all_lists(fd);
    sleep(10);
    print_hierachy(cli_get_listp()); 
    
    store = gtk_tree_store_new(LIST_N_COLUMNS, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN,
                               G_TYPE_POINTER, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN);

    printf("after gtk_tree_store_new\n");
    fill_list_store_with_data(store);
    printf("after fill_store_with_data\n");

    gtk_tree_view_set_model (GTK_TREE_VIEW(list_tree), GTK_TREE_MODEL (store));
    printf("after gtk_tree_view_set_model\n");

    g_object_unref(G_OBJECT(store));
    gtk_widget_show(list_tree);
   
    return;
}

