#include "amp.h"

/* functions used in both server and client */


GList *add_job(GList *joblist, gchar *name, gboolean random, gint position) {

    /* if called with negative value at position the job will be
     * added in the end of the list */
    struct jobqueue *jqp;

    jqp = (struct jobqueue *) g_malloc(sizeof(struct jobqueue));

    jqp -> name = g_strdup(name);
    jqp -> songqueue = NULL;
    jqp -> random = random;

    joblist = g_list_insert(joblist, (struct jobqueue *) jqp, position);
    return joblist;
}

void print_hierachy(GList *joblist) {

    if (joblist == NULL) {
        printf("No jobs in list!\n");
        return;
    }
    
    GList *liststart = joblist;

    
    while (joblist != NULL) {
        struct jobqueue *jtmp = joblist->data;
        printf("[%s] %s\n", jtmp -> random ? "RND" : "SEQ", jtmp->name);
        GList *songliststart = jtmp -> songqueue;

        while (jtmp -> songqueue != NULL) {
            struct songqueue *stmp = jtmp -> songqueue -> data;
            printf("   %s\n",stmp -> path);
            jtmp -> songqueue = g_list_next(jtmp -> songqueue);
        }
        jtmp -> songqueue = songliststart;
        joblist = g_list_next(joblist);
    }
    joblist = liststart;
}


void remove_songqueue_element(GList *joblist, gint jobnr, gint songnr) {

    GList *liststart = joblist;

    if ((joblist = g_list_nth(joblist, jobnr)) == NULL) {
        joblist = liststart;
        return;
    }

    struct jobqueue *jtmp = joblist -> data;
    GList *songliststart = jtmp -> songqueue; 


    if ((jtmp -> songqueue = g_list_nth(jtmp -> songqueue, songnr)) == NULL) {
        joblist = liststart;
        jtmp -> songqueue = songliststart;
        return;
    }

    free_songqueue_data(jtmp -> songqueue -> data);

    GList *tmp;

    tmp = g_list_remove(jtmp -> songqueue, jtmp -> songqueue -> data);
    if (songnr == 0)
        jtmp -> songqueue = tmp;
    else
        jtmp -> songqueue = songliststart;

    joblist = liststart;

}
  
void free_songqueue_data(gpointer element) {

    struct songqueue *stmp = element;
    g_free(stmp -> path);
    g_free(stmp -> info);
    g_free(stmp);
}

void free_jobqueue_data(gpointer element) {

    struct jobqueue *jtmp = element;
    g_free(jtmp -> name);
    g_free(jtmp);
}

GList *remove_jobqueue_element(GList *joblist, gint jobnr) {

    GList *liststart = joblist;

    /* go to the position of the element we want to remove */
    if ((joblist = g_list_nth(joblist, jobnr)) == NULL) {
        joblist = liststart;
        return joblist;
    }

    struct jobqueue *jtmp = joblist -> data;

    g_list_foreach(jtmp -> songqueue, (GFunc) free_songqueue_data, NULL);
    g_list_free(jtmp -> songqueue);
    free_jobqueue_data(joblist -> data);
    GList *tmp;
    tmp = g_list_remove(joblist, joblist -> data);  

    if (jobnr == 0)
        joblist = tmp;
    else
        joblist = liststart;

    return joblist;
}

struct songqueue *get_queued_song(GList *joblist, GList *highprio_list) {

    GList *liststart;

    struct songqueue *stmp;

    if (highprio_list != NULL) {
        /* things in highprio_list always go first */
        liststart = highprio_list;
        stmp = highprio_list -> data;
        gint hp_jobs = 0;
        gint total_jobs = g_list_length(highprio_list);

        while ((highprio_list != NULL) && (stmp -> played == TRUE)) {
            hp_jobs++;
            if (hp_jobs == total_jobs)
                break;
            highprio_list = g_list_next(highprio_list);
            stmp = highprio_list -> data;
        }
        if (highprio_list == NULL || hp_jobs == total_jobs) {
            /* all songs in the highprio_list are marked as played */
            highprio_list = liststart;
        }
        else {
            highprio_list = liststart;
            stmp -> played = TRUE; /* mark song as played */
            return stmp;
        }
    }

    if (joblist == NULL)
        return NULL; /* nothing to play */

    /* continue the search for a song to play in joblist */
    gint jobnr = 0;
    gboolean found;
    gint total_jobs = g_list_length(joblist);

    while (!(found = songs_to_play_in_job(joblist, jobnr))) {
        jobnr++;
        if (jobnr >= total_jobs)
            break;
    }
    if (!found) {
        joblist = liststart;
        return NULL;
    }
    /* we got a song to play in job number 'jobnr' */

    //joblist = liststart;
    liststart = joblist;

    if ((joblist = g_list_nth(joblist, jobnr)) == NULL) {
        /* this should never happen, but just in case */
        joblist = liststart;
        return NULL;
    }

    struct jobqueue *jtmp = joblist -> data;
    //struct songqueue *stmp = jtmp -> songqueue -> data;
    //GList *songliststart = jtmp -> songqueue;

    //printf("Before rand/seq play\n");
    if (!jtmp -> random) {
        /* sequence play */
        joblist = liststart;
        return (sequence_play(jtmp));
    }
    else {
        /* random play */
        printf("Random play, insert function!\n");   
        joblist = liststart;
        return (random_play(jtmp));
    }

    return NULL; /* should definately not get here */

}

struct songqueue *sequence_play(struct jobqueue *jtmp) {

    /* Gets the first element that has played marked as FALSE.
       Returns a gchar pointer to the path in the element. This
       function should only be called when we KNOW that there
       are elements that are not played.
       */

    GList *songliststart = jtmp -> songqueue;
    struct songqueue *stmp = jtmp -> songqueue -> data;

    while (jtmp -> songqueue != NULL && stmp -> played == TRUE) {
        jtmp -> songqueue = g_list_next(jtmp -> songqueue);
        stmp = jtmp -> songqueue -> data;
    }

    if (!jtmp -> songqueue) /* better than a segfault */
        return NULL;

    jtmp -> songqueue = songliststart;
    stmp -> played = TRUE; 
    return stmp;

}

struct songqueue *random_play(struct jobqueue *jtmp) {

    GList *songliststart = jtmp -> songqueue;
    struct songqueue *stmp;
    //gint32 randnr;
    struct songqueue *randelement = NULL;
    gint i = 0;
    GRand *rnd;
    
    rnd = g_rand_new();
    while (jtmp -> songqueue != NULL) {
        stmp = jtmp -> songqueue -> data;
        if (!stmp -> played) {
             i++;
             if ((g_rand_int(rnd) % i)  == 0) {
                  randelement = stmp;
             }
        }
        jtmp -> songqueue = g_list_next(jtmp -> songqueue);
    }
    jtmp -> songqueue = songliststart;
    randelement -> played = TRUE; 
    g_rand_free(rnd);
    return randelement;
}
        
    

    
gboolean songs_to_play_in_job(GList *joblist, gint jobnr) {
  
   if (joblist == NULL)
      return FALSE;

   GList *liststart = joblist;
  
   /* step through the list to the job we want to check */
   if ((joblist = g_list_nth(joblist, jobnr)) == NULL) {
      joblist = liststart;
      return FALSE;
   }
   
   /* check songqueue for specified job */
   struct jobqueue *jtmp = joblist -> data;
   struct songqueue *stmp;
   GList *songliststart = jtmp -> songqueue;
   
   while (jtmp -> songqueue != NULL) {
      stmp = jtmp -> songqueue -> data;
      if (stmp -> played == FALSE) {
         jtmp -> songqueue = songliststart;
	 joblist = liststart;
	 return TRUE;
      }
      jtmp -> songqueue = g_list_next(jtmp -> songqueue);
   }
        
   /* no non-played songs in job */
   joblist = liststart;
   jtmp -> songqueue = songliststart;

   return FALSE;
}
   
void get_current_jobstatus(GList *joblist, struct jobstatus *jstat) {
    
    struct jobqueue *jtmp = joblist -> data;

    g_list_foreach(jtmp -> songqueue, (GFunc) jobstat, jstat);
    
}
void get_current_hpstatus(GList *highprio_list, struct jobstatus *jstat) {

    g_list_foreach(highprio_list, (GFunc) jobstat, jstat);
}

void jobstat(gpointer data, gpointer jobstat) {
    
    /* ONLY called from get_current_jobstatus() and get_current_hpstatus() */
    
    struct jobstatus *jstat = jobstat;
    struct songqueue *stmp = data;
    
    jstat -> total_songs++;
    if (stmp -> played) {
        jstat -> played++;
    }
}
#if 0
void free_lists(GList *joblist, GList *hplist) {

    struct jobqueue *jtmp = NULL;
    /* free high priority list */
    printf("before hp free\n");
    g_list_foreach(hplist, (GFunc) free_songqueue_data, NULL);
    g_list_free(hplist);
    //hplist = NULL;
    printf("after hp free\n");
    
    /* free joblist and all sonqueue elements attached to it */
    while (joblist != NULL) {
        jtmp = joblist -> data;
        g_list_foreach(jtmp -> songqueue, (GFunc) free_songqueue_data, NULL);
        printf("middle\n");
        g_list_free(jtmp -> songqueue);
        free_jobqueue_data(joblist -> data);
        printf("after g_list_free jtmp -> songqueue\n");
        //free_jobqueue_data(joblist -> data);
        joblist = g_list_remove(joblist, joblist -> data);
        //joblist = g_list_next(joblist);
    }
    //free_jobqueue_data(joblist -> data);
    //g_list_foreach(joblist, (GFunc) free_jobqueue_data, NULL);
    printf("before g_list_free\n");
    g_list_free(joblist);
    if (joblist == NULL)
        printf("joblist r NULL");
    //joblist = NULL;
    return;
    
}
#endif
