#include "amp.h"

static GThread 	*cli_thread;  /* FIXME: should not be "global" for all threads, if needed */
static GThread  *play_thread;

static struct   currently_playing curr_play;

static player                  extplayer;
static player_communication    extplayer_comm;

static guint16  change_counter = 0;
static gboolean change_bool    = TRUE;

static GList *highprio_list = NULL;
static GList *joblist = NULL;

//extern struct serv_config *servcfg = NULL;
struct serv_config *servcfg;

int main() {

    gint sfd;
  
    //signal(SIGPIPE,SIG_IGN);

    servcfg = (struct serv_config *) g_malloc0(sizeof(struct serv_config));
    serv_config_read(servcfg); 
    
    if (servcfg->loglevel <= 1)
        srv_daemonfork(servcfg->logfile);
    
    srv_log(1,"Server startup");
    
    load_plugin(servcfg->player_plugin, &extplayer, &extplayer_comm);
    
    
    sfd = srv_create();
    g_thread_init(NULL);
  
    srv_waitconn(sfd); 

}

gint srv_create(void) {
  
    /* setup a socket which the clients will connect to */
    
    gint                fd;
    struct sockaddr_un 	saddr;
  
    srv_log(2, "srv_create()");
    pipe(curr_play.fd); /* need to pipe before the threads */
  
    if ((fd = socket(AF_UNIX, SOCK_STREAM, 0)) == -1) {
        perror("socket()");
        exit(-1);
    }
  
    memset(&saddr,0,sizeof(saddr));
  
    saddr.sun_family = AF_UNIX;
    strcpy(saddr.sun_path, servcfg->listen_sockfile);
  
    unlink(servcfg->listen_sockfile);
  
    if ((bind(fd, (struct sockaddr *) &saddr, sizeof (saddr))) == -1) {
        perror("bind()");
        exit(-1);
    }
    
    chmod(servcfg->listen_sockfile, SOCKFILE_PERMS);
    
    if ((listen(fd,5)) == -1) {
        perror("listen()");
    }
  
    return fd;
}

void srv_waitconn(gint sfd) {

    /* accept the new connections and start a thread */
  
    gint 		cfd;
    unsigned int	len;
    struct sockaddr_un	saddr;
  
    srv_log(2,"srv_waitconn()");
    //len = sizeof(struct sockaddr *); 
    len = sizeof(struct sockaddr_un);
    //len = sizeof(saddr);
  
    while (1) {
        /* This is where the server will block and wait for connections from
         * clients. When a new connection is established a thread will handle it.
         */
        if ((cfd = accept(sfd, (struct sockaddr *) &saddr, &len)) >= 0) { 
            cli_thread = g_thread_create(&srv_cli_handler, (gpointer) cfd, FALSE, NULL);
        }
    }
}

gpointer srv_cli_handler(gpointer data) {

    /*  when the client is connected it needs to send a
     *  connect request with its PID
     */
     
    int  		cfd;
    gchar 		buf[1024];
    fd_set        	rfds;
    pid_t		cli_pid;
    gchar               *cli_user;
    gchar               *p;
    srv_log(2, "srv_cli_handler()");
    
    cfd = GPOINTER_TO_INT(data);
    FD_ZERO(&rfds);
    FD_SET(cfd, &rfds);
  
    if ((select(cfd + 1, &rfds, NULL, NULL, NULL)) > 0) {
        if ((recv(cfd,buf,sizeof(buf)-1,0)) > 0) {     
            if ((strncmp(buf, "CONNECT", 7)) == 0) {
	        /* the client sent a connect request */
                p = strrchr(buf, ' ');
                *p = '\0';
                cli_pid = (pid_t) atoi(p+1);
                p = strchr(buf, ' ');
	        cli_user = g_strdup(p+1);
	        srv_log(3, g_strdup_printf("Connect from client with PID: %d, User: %s",cli_pid, cli_user));
	        srv_new_client(cfd, cli_user, cli_pid);
            }
        }
    }
    srv_log(1,"Client did not identify correctly.");
    return 0;
}

void srv_new_client(gint cfd, gchar *cli_user, pid_t cli_pid) {
  
    /* setup a new socketfile for the client to talk on */
   
    gint		fd;
    unsigned int	len;
    struct sockaddr_un  saddr;
    gchar   		*sockfile;
    gchar               *accept_msg;
    srv_log(2,"srv_new_client()");
    
    if ((fd = socket(AF_UNIX, SOCK_STREAM, 0)) == -1) {
        perror("socket()");
        exit(-1);
    }
  
    memset(&saddr,0,sizeof(saddr));
  
    saddr.sun_family = AF_UNIX;
    sockfile = g_strdup_printf("/tmp/cli_%s.%d", cli_user, cli_pid);
    g_free(cli_user);
    /* struct sockaddr_un { ... char sun_path[108]; ... } */
    strncpy(saddr.sun_path,sockfile,107);
    
    unlink(sockfile);
  
    if ((bind(fd, (struct sockaddr *) &saddr, sizeof (struct sockaddr_un))) == -1) {
        perror("bind()");
        exit(-1);
    }
    
    if ((listen(fd,5)) == -1) {
        perror("listen()");
    }
  
    len = sizeof(struct sockaddr_un);
    accept_msg = g_strdup_printf("ACCEPT %s",sockfile);
    write(cfd, accept_msg, strlen(accept_msg));
    g_free(accept_msg);
    close(cfd);
    srv_log(3, "Sent accept to client.");
  
    if ((cfd = accept(fd, (struct sockaddr *) &saddr, &len)) >= 0) {
    }
  
    close(fd);
  
    srv_cli_serv_talk(cfd,sockfile); 
  
    return;
}

void srv_cli_serv_talk(gint fd, gchar *sockfile) {
  
    fd_set              rfds;
    struct timeval	tv;
    gint		ret;
    gboolean            send_timeinfo = FALSE;
    struct packet       *pkt;
    time_t              t = time(NULL);
    gchar               *last = NULL;
    gint                last_changecounter = 0;
    
    
    srv_log(2, "srv_cli_serv_talk()");
  
    FD_ZERO(&rfds);
  
    while (1) {
        tv.tv_usec = SELECT_USEC_TIMEOUT;
        tv.tv_sec = 0;
        FD_SET(fd, &rfds);
     
        if ((ret = select(fd + 1, &rfds, NULL, NULL, &tv)) > 0) {
             /* packet from client arrived */
             pkt = (struct packet *) g_malloc0(sizeof(struct packet));
             if ((read(fd,&pkt->header,sizeof(struct packetheader))) <= 0)
                  break;
             if (pkt->header.length > 0) {
                  pkt->data = g_malloc0(pkt->header.length);
                  read(fd,pkt->data,pkt->header.length);
             }
         
             srv_process_pkt(fd, pkt, &send_timeinfo);
             g_free(pkt -> data);
             g_free(pkt);
        }
        if (send_timeinfo && curr_play.playing) {
            /* first we send the songqueue element of the song that
             * is playing, after that only the time in seconds.  */
            
            if (!last || last != curr_play.song->path) {
                s_send_currently_playing(fd, curr_play.song);
                last = curr_play.song->path;
            }
            else {  
                s_send_time_info(fd, curr_play.time);
            }
        }
        
        if (t+3 < time(NULL)) {
            t = time(NULL);
            if (srv_check_list_change(&last_changecounter)) {
                printf("%s: LIST HAS CHANGED\n", sockfile);
                s_send_list_changed(fd);        
                printf("after send\n");
            }
        }
    }
  
    close(fd);
    unlink(sockfile);
    g_free(sockfile);
    srv_log(3, "Client dropped connection.");  
    g_thread_exit(NULL);
}
void srv_process_pkt(gint fd, struct packet *pkt, gboolean *send_timeinfo) {

    srv_log(2, "srv_process_pkt()");

    if (servcfg->loglevel > 3) {    
        srv_log(4,"PACKET RECIVIED");
        srv_log(4, g_strdup_printf("Version: %d",pkt->header.version));
        srv_log(4, g_strdup_printf("Command: %d",pkt->header.cmd));
        srv_log(4, g_strdup_printf("Length : %d",pkt->header.length));
    }
    
    switch (pkt->header.cmd) {
        case C_PING:
            srv_log(3, "C_PING");
            s_pong(fd);
            break;
        case C_SHUTDOWN:
            srv_log(3, "C_SHUTDOWN");
            srv_shutdown();
            break;
        case C_P_PLAY:
            srv_log(3, "C_P_PLAY");
            if (curr_play.playing == FALSE) {
                curr_play.playing = TRUE;
                play_thread = g_thread_create(srv_play_loop, (gpointer) fd, FALSE, NULL);
            }
            break;
        case C_P_STOP:
            srv_log(3, "C_P_STOP");
            if (curr_play.playing == TRUE) {
                curr_play.playing = FALSE;
                (*extplayer_comm)(curr_play.fd[1],STOP,0);
            }
            break;
        case C_P_PAUSE:
            srv_log(3, "C_P_PAUSE");
            print_hierachy(joblist);
            if (curr_play.playing == TRUE) {
                (*extplayer_comm)(curr_play.fd[1],PAUSE,0);
            }
            break;
        case C_P_SEEK:
            srv_log(3, "C_P_SEEK");
            if (curr_play.playing == TRUE) {
                (*extplayer_comm)(curr_play.fd[1],SEEK, *(gint *) pkt->data);
            }
            break;
        case C_S_SET_PCM:
            srv_log(3, "C_S_SET_PCM");
            srv_set_pcm_volume(*(gint *) pkt->data);
            break;
        case C_S_REQ_TIMEINFO:
            /* send timeinfo (in seconds) from the currently_playing struct
             * to the client requesting it */
            srv_log(3, "C_S_REQ_TIMEINFO");
            *send_timeinfo = TRUE;
            break;
        case C_S_ABORT_TIMEINFO:
            srv_log(3, "C_S_ABORT_TIMEINFO");
            *send_timeinfo = FALSE;
            break;
        case C_L_ADD_JOB: {
            srv_log(3, "C_L_ADD_JOB");
            struct d_jobadd *djb = (struct d_jobadd *) pkt -> data;
            joblist = add_job(joblist, djb->name, djb->random, djb->position);
            srv_list_has_changed();
            break;
        }
        case C_L_ADD_SONG: {
            srv_log(3, "C_L_ADD_SONG");
            struct d_songadd *dsa = (struct d_songadd *) pkt -> data;
            srv_add_song_to_job(joblist, dsa->jobnr, dsa->path, dsa->position, FALSE);
            srv_list_has_changed();
            break;
        }
        case C_L_REMOVE_JOB: 
            srv_log(3, "C_L_REMOVE_JOB");
            joblist = remove_jobqueue_element(joblist, *(gint *) pkt->data);
            srv_list_has_changed();
            break;
        case C_L_REMOVE_SONG:
            srv_log(3, "C_L_REMOVE_SONG");
            srv_list_has_changed();
            break;
        case C_HPL_GET_LIST: 
            srv_log(3, "C_HPL_GET_LIST");
            s_send_hp_list(fd, highprio_list);
            break;
        case C_L_GET_JOB_LIST: 
            srv_log(3, "C_L_GET_JOB_LIST");
            s_send_joblist(fd, joblist);
            break;                       
        case C_L_GET_SONG_QUEUE: 
            srv_log(3, "C_L_GET_SONG_QUEUE");
            s_send_songqueue(fd, joblist);
            break;
        case C_HPL_ADD_SONG: {
            srv_log(3, "C_HPL_ADD_SONG");
            struct d_songadd *dsa = (struct d_songadd *) pkt -> data;
            highprio_list = srv_add_highprio_song(highprio_list, dsa -> path, dsa -> position, FALSE);
            srv_list_has_changed();
            break;
        }
    
    }

}

gpointer srv_play_loop(gpointer data) {

    struct stat file_stat;
    
    srv_log(2, "srv_play_loop()");

    /* check if plugin is loaded */
    if (!extplayer) {
        /* NOT TESTED */
        s_send_errormsg(GPOINTER_TO_INT(data), "No player-plugin loaded. Unable to play.");
        srv_log(1, "No player-plugin loaded. Unable to play.");
        curr_play.playing = FALSE;
        return 0;
    }
    /* check if a player_path is pointing to a regular file or symbolic link */
    if ((stat(servcfg->player_path, &file_stat)) == -1) {
        if (errno == ENOENT) {
            s_send_errormsg(GPOINTER_TO_INT(data), g_strdup_printf("No player found at %s. Unable to play.", servcfg->player_path));
            srv_log(1, g_strdup_printf("No player found at %s. Unable to play.", servcfg->player_path));
            return 0;
        }
    }
    else if (!(S_ISREG(file_stat.st_mode) || S_ISLNK(file_stat.st_mode))) {
        s_send_errormsg(GPOINTER_TO_INT(data),"player_path does not point on a regular file or symbolic link. Unable to play.");
        srv_log(1, "player_path does not point on a regular file or symbolic link. Unable to play.");
        return 0;
    }
    
    while (curr_play.playing) {
        if ((curr_play.song = get_queued_song(joblist, highprio_list)) != NULL) { 
            srv_list_has_changed();
            (*extplayer)(servcfg->player_path, servcfg->player_options, &curr_play);
            if (servcfg -> playlog)
                srv_playlog(servcfg->playlog, curr_play.song->path);
        }
        else    
            curr_play.playing = FALSE;
    }          
    
    srv_log(3,"Player thread done!");
    g_thread_exit(NULL);
    return 0;
}

void srv_shutdown(void) {

    /* FIXME free stuff, close fd's and wait for threads to exit */
    srv_log(1, "Server shutdown");
    
    exit(0);
}

void srv_set_pcm_volume(guint volume) {

    gint mfd;
    
    srv_log(2, "srv_set_pcm_volume()");
    if (volume > 100)
        return;

    if ((mfd = open(MIXER_DEVICE, O_WRONLY)) < 0)
        return;

    ioctl(mfd, SOUND_MIXER_WRITE_PCM, &volume);
    
    return;

}

void srv_daemonfork(gchar *logfile) {
    
    /* redirect stuff */
  
    dup2(open("/dev/null", O_RDONLY), 0);                       /* stdin  */
    
    if (logfile) {
        dup2(open(logfile, O_CREAT | O_WRONLY, LOGFILE_PERMS), 1);  /* stdout */
        dup2(open(logfile, O_CREAT | O_WRONLY, LOGFILE_PERMS), 2);  /* stderr */
    }
    else {
        srv_log(1, "No logfile specified. Redirecting to /dev/null");
        dup2(open("/dev/null", O_RDONLY), 1);
        dup2(open("/dev/null", O_RDONLY), 2);
    }
    
    if (fork())
        exit(0); /* exit parent */
    
    setsid(); /* create new session */
   
    if (fork())
        exit(0);
  
    return;
}
#if 0
void srv_startup_checks(void) {

    struct stat file_stat;

    if ((stat(servcfg->player_path, &file_stat)) == -1) {
        if (errno == ENOENT) {
            srv_log(1, g_strdup_printf("Warning: No player found at %s", servcfg->player_path));
            return;
        }
    }
    else if (file_stat.st_mode != S_IFREG | S_IFLNK) {
        srv_log(1, "Warning: player_path does not point on a regular file or symbolic link");
        return;
    }
#endif
void srv_list_has_changed(void) {

    if (change_bool == TRUE) {
        if (change_counter > 65000) {
            change_bool = FALSE;
            change_counter--; 
            return;
        }
        change_counter++;
    }
    else if (change_bool == FALSE) {
        if (change_counter < 500) {
            change_bool = TRUE;
            change_counter++;
            return;
        }
        change_counter--;
    }
}

gboolean srv_check_list_change(gint *last_value) {

    if (*last_value == 0) {
        *last_value = change_counter;
        return FALSE;
    }
    else if (*last_value != change_counter) {
        *last_value = change_counter;
        return TRUE;
    }
    
    return FALSE;
}

