#include "amp.h"

//struct serv_config *servcfg;

void serv_config_read(struct serv_config *servcfg) {

    FILE        *fp;
    gchar       *file, *dir;
    struct stat file_stat;
    
    cfg_set_default(servcfg);
    
    dir = g_strdup_printf("%s/%s", g_get_home_dir(), CONFIG_DIR);

    if ((stat(dir, &file_stat)) == -1) {
        if (errno == ENOENT) {
            srv_log(1, g_strdup_printf("Creating directory: %s", dir));
            mkdir(dir, S_IRUSR | S_IWUSR | S_IXUSR);
        }
    }
    g_free(dir);
    
    file = g_strdup_printf("%s/%s/%s", g_get_home_dir(), CONFIG_DIR, CONFIG_FILE);
    
    if ((fp = fopen(file,"r")) == NULL) {
        printf("fopen() error!\n");
        if (errno == ENOENT) {
            /* no config file found, create it */
            printf("Create a configfile!\n");    
        }
    }
    
    g_free(file);
    
    serv_config_fill_struct(fp, servcfg);

    fclose(fp);

}

void serv_config_fill_struct(FILE *fp, struct serv_config *servcfg) {

    gchar *buf;
    gchar *value;
    
    buf = (gchar *) g_malloc0(CFG_READ_BUFSIZE);
    
    while ((fgets(buf, CFG_READ_BUFSIZE-1, fp)) != NULL) {
        value = (gchar *) strchr(buf, '=');
        if (value != NULL) {
            *value = '\0';
            value++;
            value[strlen(value)-1] = '\0'; /* remove \n */
            /* now buf contains is a pointer to the option
             * and value is a pointer to that options value */
            
            if (cfg_compare(buf, "player_plugin")) 
                cfg_set_str(&servcfg->player_plugin, value);
            else if (cfg_compare(buf, "player_path"))
                cfg_set_str(&servcfg->player_path, value);
            else if (cfg_compare(buf, "player_options"))
                cfg_set_str(&servcfg->player_options, value);
            else if (cfg_compare(buf, "listen_sockfile"))
                cfg_set_str(&servcfg->listen_sockfile, value); 
            else if (cfg_compare(buf, "loglevel"))
                cfg_set_int(&servcfg->loglevel, value);
            else if (cfg_compare(buf, "playlog"))
                cfg_set_str(&servcfg->playlog, value);
            else if (cfg_compare(buf, "logfile"))
                cfg_set_str(&servcfg->logfile, value);
            
        }
        memset(buf, 0, CFG_READ_BUFSIZE);
    }    
}

gboolean cfg_compare(gchar *buf, gchar *str) {

    if ((strncmp(buf, str, strlen(str))) == 0)
        return TRUE;

    return FALSE;
}

void cfg_set_bool(gboolean *cfg, gchar *value) {

    if (cfg_compare("TRUE", value))
        *cfg = TRUE;
    else
        *cfg = FALSE;
}

void cfg_set_str(gchar **cfg, gchar *value) {

    if (*cfg)
        g_free(*cfg);

    *cfg = g_strdup(value);
}

void cfg_set_int(gint *cfg, gchar *value) {

    *cfg = atoi(value);
}


void cfg_set_default(struct serv_config *servcfg) {

    //servcfg -> player_plugin = g_strdup("~/.ajb/playerplugins/mp_pl.so");
    //servcfg -> player_path = g_strdup("/usr/local/bin/mplayer");
    //servcfg -> player_options = g_strdup("-slave");
    servcfg -> player_plugin = NULL; 
    servcfg -> player_path = NULL;
    servcfg -> player_options = NULL;
    
    servcfg -> listen_sockfile = g_strdup("/tmp/sockfile");
    servcfg -> loglevel = 1;
    servcfg -> playlog = NULL;
    servcfg -> logfile = NULL;
}
    
        

