#include "amp.h"
#include "../fileinfo/libmpg123/mpg123.h"


void fill_songinfo_struct(gchar *filename, struct songinfo *si) {

    /* try to determine what kind of file we have.. maybe not the best way to do it.. */
    srv_log(2, "fill_songinfo_struct()"); 
    gint strlength = strlen(filename);

    if ((filename[strlength-3] == 'm' || filename[strlength-3] == 'M') && (filename[strlength-2] == 'p' || filename[strlength-2] == 'P') && (filename[strlength-1] == '1' || filename[strlength-1] == '2' || filename[strlength-1] == '3')) {
        srv_log(2, "Filetype: mpX");
        fill_songinfo_struct_mpeg(filename, si);
        return;
    }
    else if ((filename[strlength-3] == 'o' || filename[strlength-3] == 'O') && (filename[strlength-2] == 'g' || filename[strlength-3] == 'G') && (filename[strlength-1] == 'g' || filename[strlength-1] == 'G')) {
        srv_log(2,"Filetype: OGG");
        return;
    }
    
    srv_log(2, g_strdup_printf("Could not determine filetype for: %s", filename));
}
        
void fill_songinfo_struct_mpeg(gchar *filename, struct songinfo *si) {
        
        /* fills the songinfo struct with various information
         * FIXME: Add id3v2 support */
    
        srv_log(2, "serv_fill_songinfo_struct_mpeg()");
        memset(si, 0, sizeof(struct songinfo));     
    
        
        si->filetype = MPEG;
        /* code below mainly from xmms */
        
        struct id3v1tag_t tag; 
        FILE *fh;
                
        if ((fh = fopen(filename, "rb")) != NULL) {

                guint32 head;
	        unsigned char tmp[4];
	        struct frame frm;
	        //gboolean id3_found = FALSE;

	        fseek(fh, -sizeof (tag), SEEK_END);
		if (fread(&tag, 1, sizeof (tag), fh) == sizeof (tag))
		{
			if ((strncmp(tag.tag, "TAG", 3)) == 0)
			{
				//id3_found = TRUE;
                                memcpy(si->title, tag.title, 30);
                                memcpy(si->artist, tag.artist, 30);
                                memcpy(si->album, tag.album, 30);
                                memcpy(si->year, tag.year, 4);
				
                                /* Check for v1.1 tags */
				if (tag.u.v1_1.__zero == 0)
				{
                                        si->trackno = tag.u.v1_1.track_number;
                                        memcpy(si->comment, tag.u.v1_1.comment, 28);
				}
				else
				{
                                        memcpy(si->comment, tag.u.v1_0.comment, 30);
                                }
				if (tag.genre <= GENRE_MAX)
                                    memcpy(si->genre, (gchar *) mpg123_id3_genres[tag.genre], 60);
                                
			}
		}

                rewind(fh);
		
                if (fread(tmp, 1, 4, fh) != 4)
		{
			fclose(fh);
			return;
		}
		head = ((guint32) tmp[0] << 24) | ((guint32) tmp[1] << 16) | ((guint32) tmp[2] << 8) | (guint32) tmp[3];
		while (!mpg123_head_check(head))
		{
			head <<= 8;
			if (fread(tmp, 1, 1, fh) != 1)
			{
				fclose(fh);
				return;
			}
			head |= tmp[0];
		}
		if (mpg123_decode_header(&frm, head))
		{
			guchar *buf;
			gdouble tpf;
			gint pos;
			XHEADDATA xing_header;
			//guint32 num_frames;
                        
			buf = g_malloc(frm.framesize + 4);
			fseek(fh, -4, SEEK_CUR);
			fread(buf, 1, frm.framesize + 4, fh);
			tpf = mpg123_compute_tpf(&frm);
			pos = ftell(fh);
			fseek(fh, 0, SEEK_END);

                        if (mpg123_get_xing_header(&xing_header, buf))
			{
                                //num_frames = xing_header.frames;
				si->vbr = TRUE;
                                si->bitrate = (gint) ((xing_header.bytes * 8) / (tpf * xing_header.frames * 1000));
                                si->samplerate = xing_header.samprate;
                        }
			else
			{
                                //num_frames = ((ftell(fh) - pos - (id3_found ? 128 : 0)) / mpg123_compute_bpf(&frm)) + 1;
				si->vbr = FALSE;
                                si->bitrate = tabsel_123[frm.lsf][frm.lay - 1][frm.bitrate_index];
                                si->samplerate = mpg123_freqs[frm.sampling_frequency];
                        }

                        gint time=0;
                        si->bytelength = ftell(fh);
                        time = mpg123_get_song_time(fh);
                        si->minutes = time / 1000 / 60;
                        si->seconds = (time/1000) % 60;
                        g_free(buf);
		}
                fclose(fh);
	}
}

